option(USE_BOOST_RC "Use Boost 1.34.0 instead of Boost 1.33.1" NO)

set(boostVersion "1_33_1")
set(asioVersion  "0_3_7")
if (USE_BOOST_RC)
    set(boostVersion "1_34_0")
endif (USE_BOOST_RC)

set(externalDir     "${CMAKE_SOURCE_DIR}/external")
set(boostDir        "boost_${boostVersion}")
set(asioDir         "boost_asio_${asioVersion}")
set(boostIncludeDir "${externalDir}/${boostDir}")
set(asioIncludeDir  "${externalDir}/${asioDir}")
set(packageDir      "${externalDir}/packages")
set(sfMirror        "http://easynews.dl.sourceforge.net")

set(BOOST_INCLUDE_DIR ${boostIncludeDir} ${asioIncludeDir} CACHE INTERNAL "")

#---
# Download the Boost package
#---

set(packageName "boost_${boostVersion}.zip")
set(packageUrl  "${sfMirror}/sourceforge/boost")
set(packageMd5  "99bd603090a07a80916deb501a2e1f07")

if (USE_BOOST_RC)
    set(packageMd5 "24a6bcd308c45473a2d125b932dfd328")
endif (USE_BOOST_RC)

set(downloadTarget "${packageDir}/.downloaded_${packageName}")
add_custom_command(
    OUTPUT  ${downloadTarget}
    COMMAND ${CMAKE_SOURCE_DIR}/scripts/downloadPackage.py
    ARGS    ${packageName} ${packageUrl} ${packageDir} ${packageMd5}
)

#---
# Unbundle the Boost package
#---

set(boostUnbundleTarget "${externalDir}/.unbundled_${boostDir}")
add_custom_command(
    OUTPUT  ${boostUnbundleTarget}
    COMMAND ${CMAKE_SOURCE_DIR}/scripts/unbundlePackage.py
    ARGS    "${packageDir}/${packageName}" ${boostIncludeDir}
    DEPENDS ${downloadTarget}
)

#---
# Download the ASIO package
#---

set(packageName "${asioDir}.zip")
set(packageUrl  "${sfMirror}/sourceforge/asio")
set(packageMd5  "dc98de177babd9ea0b305631a0fd8a15")

set(downloadTarget "${packageDir}/.downloaded_${packageName}")
add_custom_command(
    OUTPUT  ${downloadTarget}
    COMMAND ${CMAKE_SOURCE_DIR}/scripts/downloadPackage.py
    ARGS    ${packageName} ${packageUrl} ${packageDir} ${packageMd5}
)

#---
# Unbundle the ASIO package
#---

set(asioUnbundleTarget "${externalDir}/.unbundled_${asioDir}")
add_custom_command(
    OUTPUT  ${asioUnbundleTarget}
    COMMAND ${CMAKE_SOURCE_DIR}/scripts/unbundlePackage.py
    ARGS    "${packageDir}/${packageName}" ${asioIncludeDir}
    DEPENDS ${downloadTarget}
)

add_custom_target(boost-headers DEPENDS ${boostUnbundleTarget} ${asioUnbundleTarget})

#-----------------------------------------------------------------------------
macro(__boostLibrary project)
    set(fullSourceList)
    foreach (sourceFile ${ARGN})
        set(fullSourceFile "${boostIncludeDir}/libs/${project}/src/${sourceFile}")
        set(fullSourceList ${fullSourceList} ${fullSourceFile})
        set_source_files_properties(${fullSourceFile} PROPERTIES GENERATED TRUE)
    endforeach (sourceFile)
    
    include_directories(${boostIncludeDir})
    add_library(${PROJECT_NAME} ${fullSourceList})
    add_dependencies(${PROJECT_NAME} boost-headers)
endmacro(__boostLibrary)

#---
# Filesystem
#---

project(boost-filesystem)

if (USE_BOOST_RC)
    set(sourceFiles
        exception.cpp
        operations.cpp
        path.cpp
        portability.cpp
        utf8_codecvt_facet.cpp
    )
else (USE_BOOST_RC)
    set(sourceFiles
        convenience.cpp
        exception.cpp
        operations_posix_windows.cpp
        path_posix_windows.cpp
    )
endif (USE_BOOST_RC)

__boostLibrary(filesystem ${sourceFiles})

#---
# Iostreams
#---

project(boost-iostreams)
include_directories(${BZIP2_INCLUDE_DIR} ${ZLIB_INCLUDE_DIR})

if (WIN32)
    add_definitions(-DBOOST_THREAD_BUILD_LIB)
endif (WIN32)

__boostLibrary(iostreams
    bzip2.cpp
    file_descriptor.cpp
    mapped_file.cpp
    zlib.cpp
)

#---
# Unit Test Framework
#---

project(boost-unitTestFramework)

set(sourceFiles
    compiler_log_formatter.cpp
    execution_monitor.cpp
    framework.cpp
    plain_report_formatter.cpp
    progress_monitor.cpp
    results_collector.cpp
    results_reporter.cpp
    test_tools.cpp
    unit_test_log.cpp
    unit_test_main.cpp
    unit_test_monitor.cpp
    unit_test_parameters.cpp
    unit_test_suite.cpp
    xml_log_formatter.cpp
    xml_report_formatter.cpp
)

if (USE_BOOST_RC)
    set(sourceFiles ${sourceFiles}
        exception_safety.cpp
        interaction_based.cpp
        logged_expectations.cpp
    )
endif (USE_BOOST_RC)

__boostLibrary(test ${sourceFiles})

#---
# Thread
#---

project(boost-thread)

if (WIN32)
    add_definitions(-DBOOST_THREAD_BUILD_LIB)
endif (WIN32)

__boostLibrary(thread
    barrier.cpp
    condition.cpp
    exceptions.cpp
    mutex.cpp
    once.cpp
    recursive_mutex.cpp
    thread.cpp
    tss.cpp
    tss_dll.cpp
    tss_hooks.cpp
    tss_pe.cpp
    xtime.cpp
)

#---
# Regex
#---

project(boost-regex)
__boostLibrary(regex
    c_regex_traits.cpp
    cpp_regex_traits.cpp
    cregex.cpp
    fileiter.cpp
    icu.cpp
    instances.cpp
    posix_api.cpp
    regex.cpp
    regex_debug.cpp
    regex_raw_buffer.cpp
    regex_traits_defaults.cpp
    static_mutex.cpp
    usinstances.cpp
    w32_regex_traits.cpp
    wc_regex_traits.cpp
    wide_posix_api.cpp
    winstances.cpp
)

#---
# Signals
#---

project(boost-signals)
__boostLibrary(signals
    connection.cpp
    named_slot_map.cpp
    signal_base.cpp
    slot.cpp
    trackable.cpp
)

#---
# Program Options
#---

project(boost-program_options)
__boostLibrary(program_options
    cmdline.cpp
    config_file.cpp
    convert.cpp
    options_description.cpp
    parsers.cpp
    positional_options.cpp
    utf8_codecvt_facet.cpp
    value_semantic.cpp
    variables_map.cpp
    winmain.cpp
)
